# Tau-Leaping Methods for Jump-Diffusion

Tau-leaping methods approximate jump processes by "leaping" over multiple potential jump events in a single time step. These methods are essential for efficiently simulating systems with jump-diffusion processes.

## Tau-Leaping Methods

### TauLeaping - Basic Tau-Leaping
```@docs
TauLeaping
```

### CaoTauLeaping - Cao's Tau-Leaping  
```@docs
CaoTauLeaping
```

## Understanding Jump-Diffusion Processes

Jump-diffusion processes combine:
1. **Continuous diffusion**: Standard Brownian motion terms
2. **Jump processes**: Discontinuous jumps at random times

General form:
```
dX = μ(X,t)dt + σ(X,t)dW + ∫ h(X-,z)Ñ(dt,dz)
```

Where:
- μ(X,t)dt: Drift term
- σ(X,t)dW: Diffusion term  
- Ñ(dt,dz): Compensated random measure (jumps)

## When to Use Tau-Leaping

**Appropriate for:**
- Systems with many small jumps
- When exact jump simulation is computationally prohibitive
- Chemical reaction networks
- Population models with birth-death processes
- Financial models with rare events

**Not appropriate for:**
- Systems dominated by large, infrequent jumps
- When exact jump timing is critical
- Small systems where exact methods are feasible

## Method Characteristics

### TauLeaping:
- Basic tau-leaping approximation
- Fixed tau approach
- Good for initial exploration

### CaoTauLeaping:
- Adaptive tau selection
- More sophisticated error control
- Better for production simulations

## Configuration

Tau-leaping methods require:
1. **Jump rate functions**: λ(X,t) for each reaction/jump type
2. **Jump effects**: How state changes with each jump
3. **Tau selection**: Time step size strategy

```julia
# Basic setup
prob = JumpProblem(base_problem, aggregator, jumps...)
sol = solve(prob, TauLeaping())

# With adaptive tau
sol = solve(prob, CaoTauLeaping(), tau_tol=0.01)
```

## Accuracy Considerations

**Tau-leaping approximation quality depends on:**
- Jump frequency vs. tau size
- State change magnitude per jump
- System stiffness
- Error tolerance requirements

**Rule of thumb:** Tau should be small enough that jump rates don't change significantly over [t, t+tau].

## Alternative Approaches

**If tau-leaping is inadequate:**
1. **Exact methods**: Gillespie algorithm for small systems
2. **Hybrid methods**: Combine exact and approximate regions
3. **Moment closure**: For statistical properties only
4. **Piecewise deterministic**: For systems with rare jumps

## Performance Tips

1. **Vectorize jump computations** when possible
2. **Use sparse representations** for large systems
3. **Tune tau carefully** - too large gives poor accuracy, too small is inefficient
4. **Monitor jump frequencies** to validate approximation

## Integration with DifferentialEquations.jl

```julia
using DifferentialEquations, StochasticDiffEq

# Define base SDE
function drift!(du, u, p, t)
    # Continuous drift
end

function diffusion!(du, u, p, t)  
    # Continuous diffusion
end

# Define jumps
jump1 = ConstantRateJump(rate1, affect1!)
jump2 = VariableRateJump(rate2, affect2!)

# Combine into jump-diffusion problem
sde_prob = SDEProblem(drift!, diffusion!, u0, tspan)
jump_prob = JumpProblem(sde_prob, Direct(), jump1, jump2)

# Solve with tau-leaping
sol = solve(jump_prob, TauLeaping())
```

## References
- Gillespie, D.T., "Approximate accelerated stochastic simulation of chemically reacting systems"
- Cao, Y., Gillespie, D.T., Petzold, L.R., "Efficient step size selection for the tau-leaping method"